"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useAnalytics = exports.createClient = exports.AnalyticsProvider = void 0;

var _react = _interopRequireWildcard(require("react"));

var _constants = require("./constants");

var _logger = require("./logger");

var _analytics = require("./analytics");

var _storage = require("./storage");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

const createClient = config => {
  const logger = (config === null || config === void 0 ? void 0 : config.logger) || (0, _logger.createLogger)();

  if (typeof (config === null || config === void 0 ? void 0 : config.debug) === 'boolean') {
    if (config.debug) {
      logger.enable();
    } else {
      logger.disable();
    }
  }

  const clientConfig = { ..._constants.defaultConfig,
    ...config
  };
  const segmentStore = new _storage.SovranStorage({
    storeId: config.writeKey,
    storePersistor: config.storePersistor,
    storePersistorSaveDelay: config.storePersistorSaveDelay
  });
  const client = new _analytics.SegmentClient({
    config: clientConfig,
    logger,
    store: segmentStore
  }); // We don't await the client to be initialized to let the user start attaching plugins and queueing events
  // The client will handle initialization in the background and send events when it is ready
  // To subscribe to the client being fully initialized use client.isReady.onChange()

  void client.init();
  return client;
};

exports.createClient = createClient;
const Context = /*#__PURE__*/(0, _react.createContext)(null);

const AnalyticsProvider = _ref => {
  let {
    client,
    children
  } = _ref;

  if (!client) {
    return null;
  }

  return /*#__PURE__*/_react.default.createElement(Context.Provider, {
    value: client
  }, children);
};

exports.AnalyticsProvider = AnalyticsProvider;

const useAnalytics = () => {
  const client = (0, _react.useContext)(Context);
  return _react.default.useMemo(() => {
    if (!client) {
      console.error('Segment client not configured!', 'To use the useAnalytics() hook, pass an initialized Segment client into the AnalyticsProvider');
    }

    return {
      screen: async function () {
        for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
          args[_key] = arguments[_key];
        }

        return client === null || client === void 0 ? void 0 : client.screen(...args);
      },
      track: async function () {
        for (var _len2 = arguments.length, args = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
          args[_key2] = arguments[_key2];
        }

        return client === null || client === void 0 ? void 0 : client.track(...args);
      },
      identify: async function () {
        for (var _len3 = arguments.length, args = new Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
          args[_key3] = arguments[_key3];
        }

        return client === null || client === void 0 ? void 0 : client.identify(...args);
      },
      flush: async () => client === null || client === void 0 ? void 0 : client.flush(),
      group: async function () {
        for (var _len4 = arguments.length, args = new Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
          args[_key4] = arguments[_key4];
        }

        return client === null || client === void 0 ? void 0 : client.group(...args);
      },
      alias: async function () {
        for (var _len5 = arguments.length, args = new Array(_len5), _key5 = 0; _key5 < _len5; _key5++) {
          args[_key5] = arguments[_key5];
        }

        return client === null || client === void 0 ? void 0 : client.alias(...args);
      },
      reset: async function () {
        for (var _len6 = arguments.length, args = new Array(_len6), _key6 = 0; _key6 < _len6; _key6++) {
          args[_key6] = arguments[_key6];
        }

        return client === null || client === void 0 ? void 0 : client.reset(...args);
      }
    };
  }, [client]);
};

exports.useAnalytics = useAnalytics;
//# sourceMappingURL=client.js.map