"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.TimerFlushPolicy = void 0;

var _types = require("./types");

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * A Timer based flush policy.
 *
 * Flushes events on an interval.
 */
class TimerFlushPolicy extends _types.FlushPolicyBase {
  startTimer() {
    clearTimeout(this.flushTimeout);
    this.flushTimeout = setTimeout(() => {
      this.shouldFlush.value = true;
    }, this.interval);
  }
  /**
   * @param interval interval to flush in milliseconds
   */


  constructor(interval) {
    super();

    _defineProperty(this, "flushTimeout", void 0);

    _defineProperty(this, "interval", void 0);

    this.interval = interval;
  }

  start() {
    this.startTimer();
  }

  end() {
    clearTimeout(this.flushTimeout);
  }

  onEvent(_event) {
    // Reset interval
    this.startTimer();
  }

  reset() {
    super.reset();
    this.startTimer();
  }

}

exports.TimerFlushPolicy = TimerFlushPolicy;
//# sourceMappingURL=timer-flush-policy.js.map