function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import { DestinationPlugin } from '../plugin';
import { PluginType } from '../types';
import { chunk } from '../util';
import { uploadEvents } from '../api';
import { DestinationMetadataEnrichment } from './DestinationMetadataEnrichment';
import { QueueFlushingPlugin } from './QueueFlushingPlugin';
import { defaultApiHost } from '../constants';
import { checkResponseForErrors, translateHTTPError } from '../errors';
import { defaultConfig } from '../constants';
const MAX_EVENTS_PER_BATCH = 100;
const MAX_PAYLOAD_SIZE_IN_KB = 500;
export const SEGMENT_DESTINATION_KEY = 'Segment.io';
export class SegmentDestination extends DestinationPlugin {
  constructor() {
    super(...arguments);

    _defineProperty(this, "type", PluginType.destination);

    _defineProperty(this, "key", SEGMENT_DESTINATION_KEY);

    _defineProperty(this, "apiHost", void 0);

    _defineProperty(this, "isReady", false);

    _defineProperty(this, "sendEvents", async events => {
      var _this$analytics;

      if (!this.isReady) {
        // We're not sending events until Segment has loaded all settings
        return Promise.resolve();
      }

      if (events.length === 0) {
        return Promise.resolve();
      }

      const config = ((_this$analytics = this.analytics) === null || _this$analytics === void 0 ? void 0 : _this$analytics.getConfig()) ?? defaultConfig;
      const chunkedEvents = chunk(events, config.maxBatchSize ?? MAX_EVENTS_PER_BATCH, MAX_PAYLOAD_SIZE_IN_KB);
      let sentEvents = [];
      let numFailedEvents = 0;
      await Promise.all(chunkedEvents.map(async batch => {
        try {
          const res = await uploadEvents({
            writeKey: config.writeKey,
            url: this.getEndpoint(),
            events: batch
          });
          checkResponseForErrors(res);
          sentEvents = sentEvents.concat(batch);
        } catch (e) {
          var _this$analytics2, _this$analytics3;

          (_this$analytics2 = this.analytics) === null || _this$analytics2 === void 0 ? void 0 : _this$analytics2.reportInternalError(translateHTTPError(e));
          (_this$analytics3 = this.analytics) === null || _this$analytics3 === void 0 ? void 0 : _this$analytics3.logger.warn(e);
          numFailedEvents += batch.length;
        } finally {
          await this.queuePlugin.dequeue(sentEvents);
        }
      }));

      if (sentEvents.length) {
        if (config.debug === true) {
          var _this$analytics4;

          (_this$analytics4 = this.analytics) === null || _this$analytics4 === void 0 ? void 0 : _this$analytics4.logger.info(`Sent ${sentEvents.length} events`);
        }
      }

      if (numFailedEvents) {
        var _this$analytics5;

        (_this$analytics5 = this.analytics) === null || _this$analytics5 === void 0 ? void 0 : _this$analytics5.logger.error(`Failed to send ${numFailedEvents} events.`);
      }

      return Promise.resolve();
    });

    _defineProperty(this, "queuePlugin", new QueueFlushingPlugin(this.sendEvents));
  }

  getEndpoint() {
    var _this$analytics6;

    const config = (_this$analytics6 = this.analytics) === null || _this$analytics6 === void 0 ? void 0 : _this$analytics6.getConfig();
    return (config === null || config === void 0 ? void 0 : config.proxy) ?? this.apiHost ?? defaultApiHost;
  }

  configure(analytics) {
    super.configure(analytics); // Enrich events with the Destination metadata

    this.add(new DestinationMetadataEnrichment(SEGMENT_DESTINATION_KEY));
    this.add(this.queuePlugin);
  } // We block sending stuff to segment until we get the settings


  update(settings, _type) {
    const segmentSettings = settings.integrations[this.key];

    if ((segmentSettings === null || segmentSettings === void 0 ? void 0 : segmentSettings.apiHost) !== undefined && (segmentSettings === null || segmentSettings === void 0 ? void 0 : segmentSettings.apiHost) !== null) {
      this.apiHost = `https://${segmentSettings.apiHost}/b`;
    }

    this.isReady = true;
  }

  execute(event) {
    // Execute the internal timeline here, the queue plugin will pick up the event and add it to the queue automatically
    const enrichedEvent = super.execute(event);
    return enrichedEvent;
  }

  async flush() {
    return this.queuePlugin.flush();
  }

}
//# sourceMappingURL=SegmentDestination.js.map