function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import { PluginType, UpdateType } from './types';
import { getAllPlugins } from './util';
import { ErrorType, SegmentError } from './errors';
const PLUGIN_ORDER = [PluginType.before, PluginType.enrichment, PluginType.destination, PluginType.after];
export class Timeline {
  constructor() {
    _defineProperty(this, "plugins", {});
  }

  add(plugin) {
    var _plugin$analytics, _plugin$analytics2;

    const {
      type
    } = plugin;

    if (this.plugins[type]) {
      var _this$plugins$type;

      (_this$plugins$type = this.plugins[type]) === null || _this$plugins$type === void 0 ? void 0 : _this$plugins$type.push(plugin);
    } else {
      this.plugins[type] = [plugin];
    }

    const settings = (_plugin$analytics = plugin.analytics) === null || _plugin$analytics === void 0 ? void 0 : _plugin$analytics.settings.get();
    let hasInitialSettings = false;

    if (settings !== undefined) {
      plugin.update({
        integrations: settings
      }, UpdateType.initial);
      hasInitialSettings = true;
    }

    (_plugin$analytics2 = plugin.analytics) === null || _plugin$analytics2 === void 0 ? void 0 : _plugin$analytics2.settings.onChange(newSettings => {
      if (newSettings !== undefined) {
        plugin.update({
          integrations: newSettings
        }, hasInitialSettings ? UpdateType.refresh : UpdateType.initial);
        hasInitialSettings = true;
      }
    });
  }

  remove(plugin) {
    const plugins = this.plugins[plugin.type];

    if (plugins) {
      const index = plugins.findIndex(f => f === plugin);

      if (index > -1) {
        plugins.splice(index, 1);
      }
    }
  }

  apply(closure) {
    getAllPlugins(this).forEach(plugin => closure(plugin));
  }

  async process(incomingEvent) {
    let result = incomingEvent;

    for (const key of PLUGIN_ORDER) {
      const pluginResult = await this.applyPlugins({
        type: key,
        event: result
      });

      if (key !== PluginType.destination) {
        if (result === undefined) {
          return;
        } else {
          result = pluginResult;
        }
      }
    }

    return result;
  }

  async applyPlugins(_ref) {
    let {
      type,
      event
    } = _ref;
    let result = event;
    const plugins = this.plugins[type];

    if (plugins) {
      for (const plugin of plugins) {
        if (result) {
          try {
            const pluginResult = plugin.execute(result); // Each destination is independent from each other, so we don't roll over changes caused internally in each one of their processing

            if (type !== PluginType.destination) {
              result = await pluginResult;

              if (result === undefined) {
                break;
              }
            } else {
              await pluginResult;
            }
          } catch (error) {
            var _plugin$analytics3, _plugin$analytics4;

            (_plugin$analytics3 = plugin.analytics) === null || _plugin$analytics3 === void 0 ? void 0 : _plugin$analytics3.reportInternalError(new SegmentError(ErrorType.PluginError, JSON.stringify(error), error));
            (_plugin$analytics4 = plugin.analytics) === null || _plugin$analytics4 === void 0 ? void 0 : _plugin$analytics4.logger.warn(`Destination ${plugin.key} failed to execute: ${JSON.stringify(error)}`);
          }
        }
      }
    }

    return result;
  }

}
//# sourceMappingURL=timeline.js.map